package java.net;

import java.util.Iterator;
import java.util.Collections;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.concurrent.locks.ReentrantLock;
import java.util.Map;
import java.util.List;

class InMemoryCookieStore implements CookieStore
{
    private List<HttpCookie> cookieJar;
    private Map<String, List<HttpCookie>> domainIndex;
    private Map<URI, List<HttpCookie>> uriIndex;
    private ReentrantLock lock;
    
    public InMemoryCookieStore() {
        this.cookieJar = null;
        this.domainIndex = null;
        this.uriIndex = null;
        this.lock = null;
        this.cookieJar = new ArrayList();
        this.domainIndex = new HashMap();
        this.uriIndex = new HashMap();
        this.lock = new ReentrantLock(false);
    }
    
    @Override
    public void add(URI uri, HttpCookie httpCookie) {
        if (httpCookie == null) {
            throw new NullPointerException("cookie is null");
        }
        this.lock.lock();
        try {
            this.cookieJar.remove(httpCookie);
            if (httpCookie.getMaxAge() != 0L) {
                this.cookieJar.add(httpCookie);
                if (httpCookie.getDomain() != null) {
                    this.addIndex(this.domainIndex, httpCookie.getDomain(), httpCookie);
                }
                if (uri != null) {
                    this.addIndex(this.uriIndex, this.getEffectiveURI(uri), httpCookie);
                }
            }
        }
        finally {
            this.lock.unlock();
        }
        return;
    }
    
    @Override
    public List<HttpCookie> get(URI uri) {
        if (uri == null) {
            throw new NullPointerException("uri is null");
        }
        ArrayList<HttpCookie> list = new ArrayList();
        int equalsIgnoreCase = "https".equalsIgnoreCase(uri.getScheme());
        this.lock.lock();
        try {
            this.getInternal1(list, this.domainIndex, uri.getHost(), (boolean)equalsIgnoreCase);
            this.getInternal2(list, this.uriIndex, this.getEffectiveURI(uri), (boolean)equalsIgnoreCase);
        }
        finally {
            this.lock.unlock();
        }
        return list;
    }
    
    @Override
    public List<HttpCookie> getCookies() {
        this.lock.lock();
        List<Object> unmodifiableList;
        try {
            Iterator<HttpCookie> iterator = this.cookieJar.iterator();
            while (iterator.hasNext()) {
                if (!((HttpCookie)iterator.next()).hasExpired()) {
                    continue;
                }
                iterator.remove();
            }
            Collections.unmodifiableList(this.cookieJar);
        }
        finally {
            unmodifiableList = Collections.unmodifiableList(this.cookieJar);
            this.lock.unlock();
        }
        return unmodifiableList;
    }
    
    @Override
    public List<URI> getURIs() {
        ArrayList<Object> list = new ArrayList();
        this.lock.lock();
        try {
            Iterator<URI> iterator = this.uriIndex.keySet().iterator();
            while (iterator.hasNext()) {
                List<Object> list2 = (List)this.uriIndex.get((Object)iterator.next());
                if (list2 == null || list2.size() == 0) {
                    iterator.remove();
                }
                else {
                    continue;
                }
            }
        }
        finally {
            list.addAll(this.uriIndex.keySet());
            this.lock.unlock();
        }
        return list;
    }
    
    @Override
    public boolean remove(URI uri, HttpCookie httpCookie) {
        if (httpCookie == null) {
            throw new NullPointerException("cookie is null");
        }
        boolean remove = false;
        this.lock.lock();
        try {
            remove = this.cookieJar.remove(httpCookie);
        }
        finally {
            this.lock.unlock();
        }
        return remove;
    }
    
    @Override
    public boolean removeAll() {
        this.lock.lock();
        try {
            if (this.cookieJar.isEmpty()) {
                return false;
            }
            this.cookieJar.clear();
            this.domainIndex.clear();
            this.uriIndex.clear();
        }
        finally {
            this.lock.unlock();
        }
        return true;
    }
    
    private boolean netscapeDomainMatches(String anotherString, String s) {
        if (anotherString == null || s == null) {
            return false;
        }
        int equalsIgnoreCase = ".local".equalsIgnoreCase(anotherString);
        int n = anotherString.indexOf(46);
        if (n == 0) {
            n = anotherString.indexOf(46, 1);
        }
        if (equalsIgnoreCase == 0 && (n == -1 || n == anotherString.length() - 1)) {
            return false;
        }
        if (s.indexOf(46) == -1 && equalsIgnoreCase) {
            return true;
        }
        int n2 = s.length() - anotherString.length();
        if (n2 == 0) {
            return s.equalsIgnoreCase(anotherString);
        }
        if (n2 > 0) {
            s.substring(0, n2);
            return s.substring(n2).equalsIgnoreCase(anotherString);
        }
        return n2 == -1 && anotherString.charAt(0) == '.' && s.equalsIgnoreCase(anotherString.substring(1));
    }
    
    private void getInternal1(List<HttpCookie> list, Map<String, List<HttpCookie>> map, String s, boolean b) {
        ArrayList<HttpCookie> list2 = new ArrayList();
        for (Map.Entry entry : map.entrySet()) {
            String s2 = (String)entry.getKey();
            List<Object> list3 = (List)entry.getValue();
            for (HttpCookie httpCookie : list3) {
                if ((httpCookie.getVersion() == 0 && this.netscapeDomainMatches(s2, s)) || (httpCookie.getVersion() == 1 && HttpCookie.domainMatches(s2, s))) {
                    if (this.cookieJar.indexOf(httpCookie) != -1) {
                        if (!httpCookie.hasExpired()) {
                            if ((b || !httpCookie.getSecure()) && !list.contains(httpCookie)) {
                                list.add(httpCookie);
                            }
                            else {
                                continue;
                            }
                        }
                        else {
                            list2.add(httpCookie);
                        }
                    }
                    else {
                        list2.add(httpCookie);
                    }
                }
                else {
                    continue;
                }
            }
            for (HttpCookie httpCookie2 : list2) {
                list3.remove(httpCookie2);
                this.cookieJar.remove(httpCookie2);
            }
            list2.clear();
        }
    }
    
    private <T> void getInternal2(List<HttpCookie> list, Map<T, List<HttpCookie>> map, Comparable<T> comparable, boolean b) {
        for (Object next : map.keySet()) {
            if (comparable.compareTo(next) == 0) {
                List<Object> list2 = (List)map.get(next);
                if (list2 != null) {
                    Iterator<Object> iterator2 = list2.iterator();
                    while (iterator2.hasNext()) {
                        HttpCookie httpCookie = (HttpCookie)iterator2.next();
                        if (this.cookieJar.indexOf(httpCookie) != -1) {
                            if (!httpCookie.hasExpired()) {
                                if ((b || !httpCookie.getSecure()) && !list.contains(httpCookie)) {
                                    list.add(httpCookie);
                                }
                                else {
                                    continue;
                                }
                            }
                            else {
                                iterator2.remove();
                                this.cookieJar.remove(httpCookie);
                            }
                        }
                        else {
                            iterator2.remove();
                        }
                    }
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
    }
    
    private <T> void addIndex(Map<T, List<HttpCookie>> map, T t, HttpCookie httpCookie) {
        if (t != null) {
            List<HttpCookie> list = (List)map.get(t);
            if (list != null) {
                list.remove(httpCookie);
                list.add(httpCookie);
            }
            else {
                ArrayList<HttpCookie> list2 = new ArrayList();
                list2.add(httpCookie);
                map.put(t, list2);
            }
        }
    }
    
    private URI getEffectiveURI(URI uri) {
        URI uri2;
        try {
            uri2 = new URI("http", uri.getHost(), null, null, null);
        }
        catch (URISyntaxException ex) {
            uri2 = uri;
        }
        return uri2;
    }
}
